@@ -39,7 +39,7 @@ from sphinx.util import ensuredir
 from sphinx.errors import ExtensionError
 from sphinx.util.nodes import make_refnode
 from sphinxcontrib.napoleon.docstring import GoogleDocstring
-from sphinxcontrib.napoleon import Config
+from sphinxcontrib.napoleon import Config, _process_docstring
 
 from .utils import transform_node, transform_string
 from .settings import API_ROOT
@@ -48,6 +48,7 @@ from .directives import RemarksDirective, TodoDirective
 from .nodes import remarks
 
 import subprocess
+import ast
 from docuploader import shell
 
 class Bcolors:
@@ -82,6 +83,8 @@ REF_PATTERN = ':(py:)?(func|class|meth|mod|ref|attr|exc):`~?[a-zA-Z0-9_\.<> ]*(\
 REF_PATTERN_LAST = '~([a-zA-Z0-9_<>]*\.)*[a-zA-Z0-9_<>]*(\(\))?'
 
 PROPERTY = 'property'
+CODEBLOCK = "code-block"
+CODE = "code"
 
 
 # Run sphinx-build with Markdown builder in the plugin.
@@ -342,6 +345,129 @@ def _extract_signature(obj_sig):
     return signature, parameters
 
 
+# Given a line containing restructured keyword, returns which keyword it is.
+def extract_keyword(line):
+    # Must be in the form of:
+    #   .. keyword::
+    # where it begind with 2 dot prefix, followed by a space, then the keyword
+    # followed by 2 collon suffix.
+    try:
+        return line[ 3 : line.index("::") ]
+    except ValueError:
+        # TODO: handle reST template.
+        if line[3] != "_":
+            raise ValueError(f"Wrong formatting enoucntered for \n{line}\n Please check the docstring.")
+        return line
+
+
+# Given lines of code, indent to left by 1 block, based on
+# amount of trailing white space of first line as 1 block.
+def indent_code_left(lines):
+    parts = lines.split("\n")
+    # Count how much leading whitespaces there are based on first line.
+    # lstrip(" ") removes all trailing whitespace from the string.
+    tab_space = len(parts[0]) - len(parts[0].lstrip(" "))
+    parts = [part[tab_space:] for part in parts]
+    return "\n".join(parts)
+
+
+def _parse_docstring_summary(summary):
+    summary_parts = []
+    attributes = []
+    attribute_type_token = ":type:"
+    keyword = name = description = var_type = ""
+
+    # We need to separate in chunks, which is defined by 3 newline breaks.
+    # Otherwise when parsing for code and blocks of stuff, we will not be able
+    # to have the entire context when just splitting by single newlines.
+    # We should fix this from the library side for consistent docstring style,
+    # rather than monkey-patching it in the plugin.
+    for part in summary.split("\n\n\n"):
+        # Don't process empty string
+        if part == "":
+            continue
+
+        # Continue adding parts for code-block.
+        if keyword and keyword in [CODE, CODEBLOCK]:
+            # If we reach the end of keyword, close up the code block.
+            if not part.startswith(" "*tab_space) or part.startswith(".."):
+                summary_parts.append("```\n")
+                keyword = ""
+
+            else:
+                if tab_space == -1:
+                    parts = [split_part for split_part in part.split("\n") if split_part]
+                    tab_space = len(parts[0]) - len(parts[0].lstrip(" "))
+                    if tab_space == 0:
+                        raise ValueError("Code in the code block should be indented. Please check the docstring.")
+                if not part.startswith(" "*tab_space):
+                    # No longer looking at code-block, reset keyword.
+                    keyword = ""
+                    summary_parts.append("```\n")
+                summary_parts.append(indent_code_left(part))
+                continue
+
+        # Attributes come in 3 parts, parse the latter two here.
+        elif keyword and keyword == ATTRIBUTE:
+            # Second part, extract the description.
+            if not found_name:
+                description = part.strip()
+                found_name = True
+                continue
+            # Third part, extract the attribute type then add the completed one
+            # set to a list to be returned. Close up as needed.
+            else:
+                if attribute_type_token in part:
+                    var_type = part.split(":type:")[1].strip()
+                keyword = ""
+                if name and description and var_type:
+                    attributes.append({
+                        "name": name,
+                        "description": description,
+                        "var_type": var_type
+                    })
+
+                else:
+                    print("Could not process the attribute. Please check the docstring in {summary}.")
+
+                continue
+
+        # Parse keywords if found.
+        if part.startswith(".."):
+            keyword = extract_keyword(part)
+            # Works for both code-block and code
+            if keyword and keyword in [CODE, CODEBLOCK]:
+                # Retrieve the language found in the format of
+                #   .. code-block:: lang
+                # {lang} is optional however.
+                language = part.split("::")[1].strip()
+                summary_parts.append(f"```{language}")
+
+                tab_space = -1
+
+            # Extract the name for attribute first.
+            elif keyword and keyword == ATTRIBUTE:
+                found_name = False
+                name = part.split("::")[1].strip()
+
+            # Reserve for additional parts
+            # elif keyword == keyword:
+            else:
+                summary_parts.append(part + "\n")
+
+        else:
+            summary_parts.append(part + "\n")
+
+    # Close up from the keyword if needed.
+    if keyword and keyword in [CODE, CODEBLOCK]:
+        # Check if it's already closed.
+        if summary_parts[-1] != "```\n":
+            summary_parts.append("```\n")
+
+    # Requires 2 newline chars to properly show on cloud site.
+    return "\n".join(summary_parts), attributes
+
+
 # Given documentation docstring, parse them into summary_info.
 def _extract_docstring_info(summary_info, summary, name):
     top_summary = ""
@@ -546,6 +672,8 @@ def _create_datam(app, cls, module, name, _type, obj, lines=None):
         lines = []
     short_name = name.split('.')[-1]
     args = []
+    # Check how many arguments are present in the function.
+    arg_count = 0
     try:
         if _type in [METHOD, FUNCTION]:
             argspec = inspect.getfullargspec(obj) # noqa
@@ -568,6 +696,9 @@ def _create_datam(app, cls, module, name, _type, obj, lines=None):
                             print(f"Could not parse argument information for {annotation}.")
                             continue
 
+            # Add up the number of arguments. `argspec.args` contains a list of
+            # all the arguments from the function.
+            arg_count += len(argspec.args)
             for arg in argspec.args:
                 arg_map = {}
                 # Ignore adding in entry for "self"
@@ -695,11 +826,12 @@ def _create_datam(app, cls, module, name, _type, obj, lines=None):
                 app.env.docfx_xrefs[xref] = ''
 
         summary = app.docfx_transform_string('\n'.join(_refact_example_in_module_summary(lines)))
-    
+
         # Extract summary info into respective sections.
         if summary:
             top_summary = _extract_docstring_info(summary_info, summary, name)
-            datam['summary'] = top_summary
+            datam['summary'], datam['attributes'] = _parse_docstring_summary(top_summary)
+
 
     # If there is no summary, add a short snippet.
     else:
@@ -708,7 +840,9 @@ def _create_datam(app, cls, module, name, _type, obj, lines=None):
     if args or sig or summary_info:
         datam['syntax'] = {}
 
-    if args:
+    # If there are well-formatted arguments or a lot of arguments we should look
+    # into, loop through what we got from the docstring.
+    if args or arg_count > 0:
         variables = summary_info['variables']
         arg_id = []
         for arg in args:
@@ -1032,18 +1166,48 @@ def pretty_package_name(package_group):
     return " ".join(capitalized_name)
 
 
+# Check is the current lines conform to markdown header format.
+def parse_markdown_header(header_line, prev_line):
+    # Markdown h1 prefix should have only 1 of '#' character followed by exactly one space.
+    h1_header_prefix = "# "
+    if h1_header_prefix in header_line and header_line.count("#") == 1:
+        # Check for proper h1 header formatting, ensure there's more than just
+        # the hashtag character, and exactly only one space after the hashtag.
+        if not header_line[header_line.index(h1_header_prefix)+2].isspace() and \
+            len(header_line) > 2:
+
+            return header_line.strip("#").strip()
+
+    elif "=" in header_line:
+        # Check if we're inspecting an empty or undefined lines.
+        if not prev_line:
+            return ""
+
+        # Check if the current line only has equal sign divider.
+        if header_line.count("=") == len(header_line.strip()):
+            # Update header to the previous line.
+            return prev_line.strip()
+
+    return ""
+
+
 # For a given markdown file, extract its header line.
 def extract_header_from_markdown(mdfile_iterator):
+    mdfile_name = mdfile_iterator.name.split("/")[-1].split(".")[0].capitalize()
+    prev_line = ""
+
     for header_line in mdfile_iterator:
+
         # Ignore licenses and other non-headers prior to the header.
-        if "#" in header_line:
-            break
+        header = parse_markdown_header(header_line, prev_line)
+        # If we've found the header, return the header.
+        if header != "":
+            return header
 
-    if header_line.count("#") != 1:
-        raise ValueError(f"The first header of {mdfile_iterator.name} is not a h1 header: {header_line}")
+        prev_line = header_line
 
-    # Extract the header name.
-    return header_line.strip("#").strip()
+    print(f"Could not find a title for {mdfile_iterator.name}. Using {mdfile_name} as the title instead.")
+    return mdfile_name
 
 
 # Given generated markdown files, incorporate them into the docfx_yaml output.
@@ -1450,6 +1614,9 @@ def setup(app):
 
     """
 
+    app.setup_extension('sphinx.ext.autodoc')
+    app.connect('autodoc-process-docstring', _process_docstring)
+
     app.add_node(remarks, html = (remarks.visit_remarks, remarks.depart_remarks))
     app.add_directive('remarks', RemarksDirective)
     app.add_directive('todo', TodoDirective)
